/*									tab:4
 * PKT_LOSS_TEST.c - periodically send a message for testing MAC functionality
 *
 * Authors:   Wei Ye (USC/ISI)
 * History:   created 5/5/2002
 *
 */

#include "tos.h"
#include "include/phy_test_msg.h"
#include "PHY_TEST_RECEIVER.h"

// Number of packets in each group. Should match that in apps/phy_test_sender/
#ifndef TST_NUM_PKTS
#define TST_NUM_PKTS 100
#endif

#define TOS_FRAME_TYPE TEST_frame
TOS_FRAME_BEGIN(TEST_frame) {
	AppPkt data; 		// message to be sent
	unsigned char numRx;  // number of succesfully received pkts
	unsigned char numLenErr;  // number of errors in length field 
	unsigned char numErrPkt;  // number of received pkts with CRC errors
	unsigned char numStart;  // number of packets that received first byte
#ifdef CARR_SEN_TEST
	unsigned char numChanIdle;
	unsigned char numChanBusy;
#endif
}
TOS_FRAME_END(TEST_frame);


char TOS_COMMAND(TEST_INIT)() {
	VAR(numRx) = 0;
	VAR(numLenErr) = 0;
	VAR(numErrPkt) = 0;
	VAR(numStart) = 0;
	TOS_CALL_COMMAND(TEST_LEDS_INIT)();
	TOS_CALL_COMMAND(TEST_SUB_INIT)();  // initialize lower components
#ifdef CARR_SEN_TEST
	VAR(numChanIdle) = 0;
	VAR(numChanBusy) = 0;
#endif
	return 1;
}


char TOS_COMMAND(TEST_START)() {
	return 1;
}


char TOS_EVENT(TEST_SUB_TX_PKT_DONE)(void* msg) {
	return 1;
}


char TOS_EVENT(TEST_SUB_START_SYM_DETECTED)() {
	// start symbol detected
	VAR(numStart)++;
//	TOS_CALL_COMMAND(TEST_LEDy_tog)();
	
	return 1;
}


void* TOS_EVENT(TEST_SUB_RX_PKT_DONE)(void* data, char error)
{
	if (data == NULL) {
		VAR(numLenErr)++;
//		TOS_CALL_COMMAND(TEST_LEDr_tog)();
		return data;
	}
	if (VAR(numRx) == 0) {
		TOS_CALL_COMMAND(TEST_LEDr_off)();
		TOS_CALL_COMMAND(TEST_LEDy_off)();
	}
	if (error)
		VAR(numErrPkt)++;
	else {
		VAR(numRx)++;
		TOS_CALL_COMMAND(TEST_LEDg_tog)();
	}
	if (((AppHeader*)data)->seqNo == TST_NUM_PKTS - 1) {  // got last pkt
		// remember result of this group of packets
		VAR(data).data[0] = VAR(numRx); // received pkts without error
		VAR(data).data[1] = VAR(numLenErr); // num of errors in length field
		VAR(data).data[2] = VAR(numErrPkt); // received pkts w/ CRC errors
		VAR(data).data[3] = VAR(numStart); // pkts whose first byte is received
#ifdef CARR_SEN_TEST
		VAR(data).data[5] = VAR(numChanIdle);
		VAR(data).data[6] = VAR(numChanBusy);
#endif
		// turn on LEDs to show result
		TOS_CALL_COMMAND(TEST_LEDy_on)();
		if (VAR(numRx) == TST_NUM_PKTS) {
			TOS_CALL_COMMAND(TEST_LEDr_on)();
			TOS_CALL_COMMAND(TEST_LEDg_on)();
		}
		VAR(numRx) = 0;
		VAR(numLenErr) = 0;
		VAR(numErrPkt) = 0;
		VAR(numStart) = 0;
#ifdef CARR_SEN_TEST
		VAR(numChanIdle) = 0;
		VAR(numChanBusy) = 0;
#endif
		// report my reception result
		TOS_CALL_COMMAND(TEST_SUB_TX_PKT)(&VAR(data), sizeof(AppPkt));
	}
	return data;
}


char TOS_EVENT(TEST_SUB_CHANNEL_IDLE)() {
	return 1;
}

char TOS_EVENT(TEST_SUB_CHANNEL_BUSY)() {
	return 1;
}

#ifdef CARR_SEN_TEST
char TOS_EVENT(TEST_SUB_CHANNEL_STATE_CHANGE)(char chanState) {
	if (chanState) { // channel changes from idle to busy
		VAR(numChanBusy)++;
	} else {  // channel changes from busy to idle
		VAR(numChanIdle)++;
	}
	return 1;
}
#endif
