/* CODEC_4B_6B.c                                            tab:4
 *
 * Copyright (c) 2002 the University of Southern California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF SOUTHERN CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF SOUTHERN CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF SOUTHERN CALIFORNIA HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 *
 * Authors:  Wei Ye (USC/ISI)
 * Date created: Sept 1, 2002
 *
 * This module implements Manchester encoding and decoding
 *
 */


#include "tos.h"
#include "CODEC_4B_6B.h"
#include "dbg.h"

#define IDLE_STATE 0
#define ENCODING_BYTE 1 
#define DECODING_BYTE_1 2
#define DECODING_BYTE_2 3 

//table for performing the encoding.
static const char PROGMEM encodeTab[] =
{0x15,
 0x31,
 0x32,
 0x23,
 0x34,
 0x25,
 0x26,
 0x07,
 0x38,
 0x29,
 0x2a,
 0x0b,
 0x2c,
 0x0d,
 0x0e,
 0x1c};

/* Frame of the component */
#define TOS_FRAME_TYPE codec_frame
TOS_FRAME_BEGIN(codec_frame) {
  char state;
  char error;
  char leftBits;
  unsigned char decodedVal;
  unsigned char nibVal;
}
TOS_FRAME_END(codec_frame);

char radio_decode_thread(char data);

char TOS_COMMAND(CODEC_INIT)(){
	VAR(state) = IDLE_STATE;
	return 1;
}

char TOS_COMMAND(CODEC_DECODE)(char data){
	char nibble;
	if(VAR(state) == IDLE_STATE){
		VAR(state) = DECODING_BYTE_1;
		nibble = (data >> 2) & 0x3f;
		VAR(leftBits) = (data << 4) & 0x30;
		VAR(error) = radio_decode_thread(nibble);
		VAR(decodedVal) = VAR(nibVal) << 4;
		return 1;
	} else if (VAR(state) == DECODING_BYTE_1) {
		VAR(state) = DECODING_BYTE_2;
		nibble = VAR(leftBits) | ((data >> 4) & 0xf);
		VAR(leftBits) = (data << 2) & 0x3c;
		VAR(error) |= radio_decode_thread(nibble);
		VAR(decodedVal) |= VAR(nibVal); 
     	TOS_SIGNAL_EVENT(CODEC_DECODE_DONE)(VAR(decodedVal), VAR(error));
		return 1;
	} else if (VAR(state) == DECODING_BYTE_2) {
		nibble = VAR(leftBits) | ((data >> 6) & 0x3);
		VAR(error) = radio_decode_thread(nibble);
		VAR(decodedVal) = VAR(nibVal) << 4;
		nibble = data & 0x3f;
		VAR(error) |= radio_decode_thread(nibble);
		VAR(decodedVal) |= VAR(nibVal); 
     	TOS_SIGNAL_EVENT(CODEC_DECODE_DONE)(VAR(decodedVal), VAR(error));
		VAR(state) = IDLE_STATE;
		return 1;
	}
	return 0;	
}


char TOS_COMMAND(CODEC_ENCODE_FLUSH)(){
	if (VAR(state) != IDLE_STATE) {
		VAR(state) = IDLE_STATE;
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)(VAR(leftBits));
	}
	return 1;
}

char TOS_COMMAND(CODEC_ENCODE)(char d){
	char high, low, value;

	low = __lpm_inline((unsigned short)encodeTab + (d & 0xf));
	high = __lpm_inline((unsigned short)encodeTab + ((d >> 4) & 0xf));
	
	if(VAR(state) == IDLE_STATE){
		VAR(state) = ENCODING_BYTE;
		value = (high << 2) | ((low >> 4) & 0x3);
		VAR(leftBits) = low << 4;
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)(value); // high byte is sent first
	} else if (VAR(state) == ENCODING_BYTE) {
		value = VAR(leftBits) | ((high >> 2) & 0xf);
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)(value);
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)((high << 6) | low);
		VAR(state) = IDLE_STATE;
	}
	return 1;
}

char radio_decode_thread(char data){
	char error = 0;
	char check = data & 0x1;
	unsigned char i;
	for (i = 1; i < 6; i++) {
		check += (data >> i) & 0x1;
	}
	if (check != 3) error = 1;
	if (data == 0x15) VAR(nibVal) = 0;
	else if(data == 0x1c) VAR(nibVal) = 0xf;
	else VAR(nibVal) = data & 0xf;
    return error;
}
